/* Custom styling for Pinecone Python SDK documentation */

/* Inter font family for better readability */
body,
div.body h1,
div.body h2,
div.body h3,
div.body h4,
div.body h5,
div.body h6,
div.admonition p.admonition-title {
    font-family: "Inter", "Helvetica Neue", "Helvetica", "Arial", sans-serif;
}

/* Blurb styling */
.blurb {
    font-size: 16px;
}

/* Remove colon after admonition titles */
p.admonition-title:after {
    content: "";
}

/* Code block caption styling */
div.code-block-caption {
    background-color: #EEE;
    border-bottom: 1px solid #CCC;
    font-size: 17px;
    padding: 10px;
}

/* Target highlighting for direct links */
dt:target {
    background-color: #E8E8E8;
}

/* Code highlight background */
.highlight {
    background-color: #F8F8F8;
}

/* Improve code block readability */
div.highlight pre {
    padding: 12px;
    font-size: 14px;
    line-height: 1.5;
}

/* Better spacing for parameters */
dl.py.class > dd,
dl.py.function > dd,
dl.py.method > dd {
    margin-left: 30px;
}

/* Improve table styling */
table.docutils {
    border-collapse: collapse;
    border: 1px solid #ddd;
}

table.docutils th {
    background-color: #f5f5f5;
    padding: 8px;
    border: 1px solid #ddd;
}

table.docutils td {
    padding: 8px;
    border: 1px solid #ddd;
}

/* Better link styling */
a.reference.external,
a.reference.internal {
    text-decoration: none;
    border-bottom: 1px dotted;
}

a.reference.external:hover,
a.reference.internal:hover {
    border-bottom: 1px solid;
}

/* Sidebar improvements */
div.sphinxsidebar {
    font-size: 14px;
}

div.sphinxsidebar h3 {
    font-size: 18px;
    margin-top: 0;
}

/* Navigation improvements */
div.sphinxsidebar ul {
    margin: 10px 0;
    padding: 0;
}

div.sphinxsidebar li {
    list-style-type: none;
    margin: 5px 0;
}

/* Code signature styling */
dl.py dt {
    background-color: #f5f5f5;
    padding: 5px 10px;
    border-left: 3px solid #2980b9;
}

/* Better spacing for method/class documentation */
dl.py {
    margin-bottom: 20px;
}

/* Improve readability of parameter lists */
dl.field-list {
    margin-top: 10px;
}

dl.field-list dt {
    font-weight: bold;
    margin-top: 5px;
}

dl.field-list dd {
    margin-left: 20px;
    margin-bottom: 10px;
}
