"""Common models shared across Inference API operations.

API Version: 2025-10
"""

from pinecone.models._base import BaseModel


class Usage(BaseModel, kw_only=True):  # pyright: ignore[reportCallIssue]
    """Token usage information for API request.

    Attributes:
        total_tokens: Total number of tokens used
    """

    total_tokens: int
